/*
	mesh_reader.c

	lecteur de fichier xxx.mesh (http://www-c.inria.fr/gamma/download/DREXEL/index1.php)

	Auteur : Florent Kaisser

	Université de Cergy-Pontoise
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "mesh_reader.h"

#define MAX_LINE 200

int print_figure(Figure *f){
	int i;
	for(i=0;i<f->nbVertex;i++)
		printf("%d(%f %f %f)\n",i,f->points[i].x,f->points[i].y,f->points[i].z);

	for(i=0;i<f->nbTriangle;i++)
		printf("%d[%d %d %d] n(%0.2f %0.2f %0.2f)\n",i,f->triangles[i].v1,f->triangles[i].v2,f->triangles[i].v3,f->triangles[i].normal.x,f->triangles[i].normal.y,f->triangles[i].normal.z);
}

static int erreur(char *msg){
	printf("erreur : %s\n",msg);
	return 0;
}






void produitVectoriel(const Point *u,const Point *v,Point *w)
{
        w->x = (u->y*v->z)-(u->z*v->y);
        w->y = (u->z*v->x)-(u->x*v->z);
        w->z = (u->x*v->y)-(u->y*v->x);
}

double norme(const Point* u)
{
        return sqrt((u->x*u->x) +(u->y*u->y) +(u->z*u->z));
}

void normalizeVecteur(Point* u)
{
        double n = norme(u);

        u->x /=n;
        u->y /=n;
        u->z /=n;
}

//w=u-v
void soustraireVecteur(const Point *u,const Point *v,Point *w)
{
        w->x=u->x-v->x;
        w->y=u->y-v->y;
        w->z=u->z-v->z;
}

//Calcule et alloue le vecteur normal de chaque facette
void calculNormal(Figure* fig)
{
        Point u,v;
        int i;


       for(i=0;i<fig->nbTriangle;i++){
		Triangle *t=fig->triangles+i;

                soustraireVecteur(fig->points+t->v2,fig->points+t->v1,&u);
                soustraireVecteur(fig->points+t->v3,fig->points+t->v1,&v);
                produitVectoriel(&u,&v,&(t->normal));
                normalizeVecteur(&(t->normal));
       }

}



static int read_line_mesh(FILE* df,char *line,int n){


	while(fgets(line,n,df)){

		if(line[0]!='#')
			return 1;

	}

	return 0;
	
}

int read_mesh_file(FILE* fd,Figure *f){
	char comment;
	int n,i;
	char ligne[MAX_LINE];
	char word[MAX_LINE];
	int version;
	Point p;
	Triangle t;

	read_line_mesh(fd,ligne,MAX_LINE);

	if(sscanf(ligne,"%s %d",word,&version)!=2)
	/*{
		printf ("version %d\n",version);
	/*}else*/ return erreur("pas de version");

	read_line_mesh(fd,ligne,MAX_LINE);

	if(sscanf(ligne,"%s",word)==1){
		read_line_mesh(fd,ligne,MAX_LINE);
		if(sscanf(ligne,"%d",&n)!=1){
		//	printf ("dimension %d\n",n);
		//else 
			return erreur("pas d'argument pour dimension \n");
		}
	}else return erreur("pas de dimension");
	
	read_line_mesh(fd,ligne,MAX_LINE);
	
	if(sscanf(ligne,"%s",word)==1){
		read_line_mesh(fd,ligne,MAX_LINE);
		if(sscanf(ligne,"%d",&n)!=1){
		//	printf ("sommet %d\n",n);
		//else 
			return erreur(" pas d'argument pour sommet \n");
		}
	}else return erreur("pas de sommet\n");

	f->nbVertex = n;
	f->points = malloc(sizeof(Point)*n) ;
	
	//printf("lecture de %d sommets\n",n);

	for(i=0;i<n;i++){
		int na;
		read_line_mesh(fd,ligne,MAX_LINE);
		if(sscanf(ligne,"%f %f %f %d",&p.x,&p.y,&p.z,&na)==4){
			//printf("lecture du sommet %d : %f %f %f\n",i,p.x,p.y,p.z);
			f->points[i]=p;
		}else return erreur("dans la lecture d'un sommet\n");
	}
	
	read_line_mesh(fd,ligne,MAX_LINE);

	if(sscanf(ligne,"%s",word)==1){
		read_line_mesh(fd,ligne,MAX_LINE);
		if(sscanf(ligne,"%d",&n)!=1)
		//	printf ("triangle %d\n",n);
		//else 
			return erreur(" pas d'argument pour triangles \n");
	}else return erreur("pas de triangles\n");

	f->nbTriangle = n;
	f->triangles = malloc(sizeof(Triangle)*n) ;
	
	//printf("lecture de %d triangles\n",n);

	for(i=0;i<n;i++){
		int na;
		read_line_mesh(fd,ligne,MAX_LINE);
		if(sscanf(ligne,"%d %d %d %d",&t.v1,&t.v2,&t.v3,&na)==4){
			//printf("lecture du triangle %d : %d %d %d\n",i,t.v1,t.v2,t.v3);
			t.v1--;t.v2--,t.v3--;
			f->triangles[i]=t;
		}else return erreur("dans la lecture d'un sommet\n");
	}

	calculNormal(f);

	return 1;
}

